<?php

/*

	Plugin Name: Syndicate Out
	Plugin URI: http://www.flutt.co.uk/development/category/wordpress-plugins/syndicate-out-wordpress-plugins/
	Version: 0.5
	Description: Syndicates posts made in any specified category to another WP blog using WordPress' built in XML-RPC functionality.
	Author: Jonathon Wardman
	Author URI: http://www.flutt.co.uk/

*/

#
#  syndicate_out.php
#
#  Created by Jonathon Wardman on 09-07-2009.
#  Copyright 2009 - 2010, Jonathon Wardman. All rights reserved.
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  You may obtain a copy of the License at:
#  http://www.gnu.org/licenses/gpl-3.0.txt
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.


	 // Nothing in this plugin works outside of the admin area, so don't bother
	 // loading it if we're not looking at the admin panel...
if ( is_admin() ) {

 /* Setup section. */

	 // Global constants and variables relating to posts...
	$postIsEdit = false;
	define('SO_OPTIONS_VERSION', '2');

	 // Register functions...
	add_action( 'admin_menu', 'syndicate_out_menu' );
	add_action( 'admin_init', 'syndicate_out_register_settings' );
	add_action( 'wp_insert_post', 'syndicate_out_post' );
	add_action( 'pre_post_update', 'syndicate_out_edit', 5 );

    // Register the plugin activation function...
	register_activation_hook( __FILE__, 'syndicate_out_activate' );

 /* Admin section. */

	 // Admin menu...
	function syndicate_out_menu() {

		add_submenu_page( 'options-general.php', 'Syndicate Out Settings', 'Syndication', 'manage_options', 'syndicate_out', 'syndicate_out_admin' );

	}

	 // Register valid admin options...
	function syndicate_out_register_settings() {
	
		register_setting( 'syndicate-out-options', 'so_options', 'syndicate_out_sanitize_options' );

	}

	 // Admin page...
	function syndicate_out_admin() {
	
		require_once( 'so-options.php' );

	}

 /* Post / action section. */
 
	 // Sanitize and organise the all settings...
	function syndicate_out_sanitize_options( $options ) {
	
	 // Delete any groups which have been flagged for deletion...
		if ( isset( $options['deletegroup'] ) ) {
			foreach ( $options['deletegroup'] AS $groupKey => $buttonValue ) {
				if ( array_key_exists( $groupKey, $options ) ) {
					unset( $options[$groupKey] );
				}
			}
			unset( $options['deletegroup'] );
		}
		
	 // Add new groups, if required...
		if ( isset( $options['addgroup'] ) ) {
			$options[] = array();
			unset( $options['addgroup'] );
		}
		
	 // Flag new rows, if required...
		if ( isset( $options['addrow'] ) ) {
			$newServerArray = $options['addrow'];
			unset( $options['addrow'] );
		}

	 // Save all group settings...
		$newOptions = array( 'group' => array() );
		foreach ( $options AS $groupId => $groupOptions ) {
		
			switch ( $groupOptions['syndicate_category'] ) {
				case 'syndication': case 'all': case 'none':
					$newOptions['group'][$groupId]['syndicate_category'] = $groupOptions['syndicate_category'];
				break;
				default:
					$newOptions['group'][$groupId]['syndicate_category'] = 'none';
				break;
			}

			if ( 'true' == $groupOptions['allcats'] ) {
				$newOptions['group'][$groupId]['category'] = -1;
			} else {
				$newOptions['group'][$groupId]['category'] = $groupOptions['category'];
			}

			foreach ( $groupOptions['servers'] AS $serverKey => $serverDetails ) {
				if ( '' != $serverDetails['server'] ) {
					$remoteServer = trim( $serverDetails['server'] );
					if ( 'http://' != substr( $remoteServer, 0, 7 ) ) {
			         $remoteServer = 'http://'.$remoteServer;
					}
					if ( '/' != substr( $remoteServer, -1 ) ) {
			         $remoteServer .= '/';
					}
					$newOptions['group'][$groupId]['servers'][$serverKey]['server'] = $remoteServer;
					$newOptions['group'][$groupId]['servers'][$serverKey]['username'] = $serverDetails['username'];
					$newOptions['group'][$groupId]['servers'][$serverKey]['password'] = $serverDetails['password'];
				}
			}
			if ( isset( $newServerArray ) && array_key_exists( $groupId, $newServerArray) ) {
				$newOptions['group'][$groupId]['servers'][($serverKey + 1)] = array();
			}
		}

		$oldOptions = get_option( 'so_options' );
		if ( isset( $oldOptions['options_version'] ) ) {
			$newOptions['options_version'] = $oldOptions['options_version'];
		} else {
			$newOptions['options_version'] = SO_OPTIONS_VERSION;
		}
		return $newOptions;

	}

	 // Carry out the syndication on post insert...
	function syndicate_out_post( $postId ) {
	
		$soOptions = get_option( 'so_options' );
		
		if ( -1 == $syndicationGroup['category'] ) {
			$activeGroups = $soOptions['group'];
		} else {
			$activeGroups = array();
			foreach ( $soOptions['group'] AS $syndicationGroup ) {
				if (  in_category( $syndicationGroup['category'], $postId ) ) {
					$activeGroups[] = $syndicationGroup;
				}
			}
		}
		
		if ( count( $activeGroups ) > 0 ) {
			if ( @include_once(  ABSPATH . WPINC . '/class-IXR.php' ) ) {

				$postData = get_post( $postId );

				if ( 'publish' == $postData->post_status ) {

					$remotePost['title'] = $postData->post_title;
					$remotePost['description'] = $postData->post_content;
					if ( $postTags = syndicate_out_get_tags( $postId ) ) {
						$keywords = array();
						foreach ( $postTags AS $postTag ) {
							$keywords[] = $postTag->name;
						}
						$remotePost['mt_keywords'] = implode( ',', $keywords );
					}
					if ( 'none' != $soOptions['syndicate_category'] ) {
						if ( 'syndication' == $soOptions['syndicate_category'] ) {
							$remotePost['categories'] = array( get_cat_name( $soOptions['category'] ) );
						} else if ( 'all' == $soOptions['syndicate_category'] ) {
							$remotePost['categories'] = array();
							$categories = get_the_category( $postId );
							foreach ( $categories AS $postCategory ) {
                        $remotePost['categories'][] = $postCategory->cat_name;
							}
						}
					}

				}
				
				if ( false === $GLOBALS['postIsEdit'] ) {
					$remotePostInformation = array( 'options_version' => SO_OPTIONS_VERSION );
					foreach ( $activeGroups AS $groupKey => $activeGroup ) {
						foreach ( $activeGroup['servers'] AS $serverKey => $serverDetails ) {
							$xmlrpc = new IXR_Client( $serverDetails['server'].'xmlrpc.php' );
							$xmlrpc->query( 'metaWeblog.newPost', 1, $serverDetails['username'], $serverDetails['password'], $remotePost, 1 );
							$remotePostInformation['group'][$groupKey][$serverKey] = $xmlrpc->getResponse();
						}
					}
					update_post_meta( $postId, '_so_remote_posts', serialize( $remotePostInformation ) );
				} else {
					$remotePostIds = unserialize( get_post_meta( $postId, '_so_remote_posts', true ) );
					if ( !isset( $remotePostIds['options_version'] ) ) {
						$newRemotePostIds = array( 'options_version' => SO_OPTIONS_VERSION );
						foreach ( $remotePostIds AS $serverKey => $remotePostId ) {
							$newRemotePostIds['group'][0][$serverKey] = $remotePostId;
						}
						$remotePostIds = $newRemotePostIds;
						update_post_meta( $postId, '_so_remote_posts', serialize( $remotePostIds ) );
					}
					foreach ( $remotePostIds['group'] AS $groupKey => $remoteServers ) {
						foreach ( $remoteServers AS $serverKey => $remotePostId ) {
							if ( is_numeric( $remotePostId ) ) {
								if ( isset( $soOptions['group'][$groupKey]['servers'][$serverKey] ) ) {
									$xmlrpc = new IXR_Client( $soOptions['group'][$groupKey]['servers'][$serverKey]['server'].'xmlrpc.php' );
									$xmlrpc->query( 'metaWeblog.editPost', $remotePostId, $soOptions['group'][$groupKey]['servers'][$serverKey]['username'], $soOptions['group'][$groupKey]['servers'][$serverKey]['password'], $remotePost, 1 );
								}
							}
						}
					}
				}

			}
		}

	}

	 // Mark edited posts as edits (not new posts)...
	function syndicate_out_edit( $postId ) {

		$postData = get_post( $postId );
		if ( 'draft' != $postData->post_status ) {
			$GLOBALS['postIsEdit'] = true;
		}

	}

	 // Get a list of tags for this post...
	function syndicate_out_get_tags( $postId ) {

		$terms = get_object_term_cache( $postId, 'post_tag' );
		if ( false === $terms ) {
			$terms = wp_get_object_terms( $postId, 'post_tag' );
		}

		if ( empty( $terms ) ) {
			return false;
		}

		return $terms;

	}

 /* Maintenance section. */

	 // Activation function.  Updates the old settings storage format to the
	 // new settings storage format...
	function syndicate_out_activate() {

		if ( $oldSettingsCategory = get_option( 'so_category' ) ) {
			$oldSettingsServer = get_option( 'so_remote_server' );
			$oldSettingsUsername = get_option( 'so_remote_username' );
			$oldSettingsPassword = get_option( 'so_remote_password' );
			$newSettings = array( 'group' => array( array( 'category' => $oldSettingsCategory,
			                                               'syndicate_category' => 'none',
			                                               'servers' => array( array( 'server' => $oldSettingsServer,
			                                                                          'username' => $oldSettingsUsername,
			                                                                          'password' => $oldSettingsPassword ) ) ) ),
			                      'options_version' => SO_OPTIONS_VERSION );
			update_option( 'so_options', $newSettings );
			delete_option( 'so_category' );
			delete_option( 'so_remote_server' );
			delete_option( 'so_remote_username' );
			delete_option( 'so_remote_password' );

		} else if ( $oldSettings = get_option( 'so_options' ) ) {
		
			if ( isset( $oldSettings['options_version'] ) ) {
				$optionsVersion = $oldSettings['options_version'];
				unset($oldSettings['options_version']);
			} else {
				$optionsVersion = 0;
			}
			switch ($optionsVersion) {
				case 0: case 1:
					$newSettings['group'][0] = $oldSettings;
				break;
			}
			$newSettings['options_version'] = SO_OPTIONS_VERSION;
			update_option( 'so_options', $newSettings );
			
		}

	}

}